"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModernEngine = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@yarnpkg/core");
const constraintUtils = tslib_1.__importStar(require("./constraintUtils"));
class ModernEngine {
    constructor(project) {
        this.project = project;
    }
    createEnvironment() {
        const workspaces = new constraintUtils.Index([`cwd`, `ident`]);
        const dependencies = new constraintUtils.Index([`type`, `ident`]);
        const result = {
            manifestUpdates: new Map(),
            reportedErrors: new Map(),
        };
        for (const workspace of this.project.workspaces) {
            const ident = core_1.structUtils.stringifyIdent(workspace.anchoredLocator);
            const manifest = workspace.manifest.exportTo({});
            const setFn = (path, value, { caller = core_1.nodeUtils.getCaller() } = {}) => {
                const normalizedPath = constraintUtils.normalizePath(path);
                const workspaceUpdates = core_1.miscUtils.getMapWithDefault(result.manifestUpdates, workspace.cwd);
                const pathUpdates = core_1.miscUtils.getMapWithDefault(workspaceUpdates, normalizedPath);
                const constraints = core_1.miscUtils.getSetWithDefault(pathUpdates, value);
                if (caller !== null) {
                    constraints.add(caller);
                }
            };
            const unsetFn = (path) => {
                return setFn(path, undefined, { caller: core_1.nodeUtils.getCaller() });
            };
            const workspaceItem = workspaces.insert({
                cwd: workspace.cwd,
                ident,
                manifest,
                set: setFn,
                unset: unsetFn,
            });
            for (const dependencyType of core_1.Manifest.allDependencies) {
                for (const descriptor of workspace.manifest[dependencyType].values()) {
                    const ident = core_1.structUtils.stringifyIdent(descriptor);
                    const deleteFn = () => {
                        setFn([dependencyType, ident], undefined, { caller: core_1.nodeUtils.getCaller() });
                    };
                    const updateFn = (range) => {
                        setFn([dependencyType, ident], range, { caller: core_1.nodeUtils.getCaller() });
                    };
                    const errorFn = (message) => {
                        core_1.miscUtils.getArrayWithDefault(result.reportedErrors, workspace.cwd).push(message);
                    };
                    dependencies.insert({
                        workspace: workspaceItem,
                        ident,
                        range: descriptor.range,
                        type: dependencyType,
                        update: updateFn,
                        delete: deleteFn,
                        error: errorFn,
                    });
                }
            }
        }
        return {
            workspaces,
            dependencies,
            result,
        };
    }
    async process() {
        const env = this.createEnvironment();
        const context = {
            Yarn: {
                workspace: filter => {
                    var _a;
                    return (_a = env.workspaces.find(filter)[0]) !== null && _a !== void 0 ? _a : null;
                },
                workspaces: filter => {
                    return env.workspaces.find(filter);
                },
                dependencies: filter => {
                    return env.dependencies.find(filter);
                },
            },
        };
        const userConfig = await this.project.loadUserConfig();
        if (!(userConfig === null || userConfig === void 0 ? void 0 : userConfig.constraints))
            return null;
        userConfig.constraints(context);
        return env.result;
    }
}
exports.ModernEngine = ModernEngine;
